@@ -1,13 +1,12 @@
-import os
 from typing import Any, Callable, Dict, List, Optional, Tuple, Union
-from urllib.parse import urljoin, urlparse
 
 import numpy as np
-
 from Orange.data import ContinuousVariable, Domain, Table, Variable
 from Orange.misc.utils.embedder_utils import EmbedderCache
+
 from orangecontrib.imageanalytics.local_embedder import LocalEmbedder
 from orangecontrib.imageanalytics.server_embedder import ServerEmbedder
+from orangecontrib.imageanalytics.utils.image_utils import extract_paths
 
 MODELS = {
     "inception-v3": {
@@ -20,7 +19,7 @@ MODELS = {
         # high for inception since it has many workers, but other embedders
         # send less images since bottleneck are workers, this way we avoid
         # ReadTimeout because of images waiting in a queue at the server
-        "batch_size": 100,
+        "batch_size": 500,
     },
     "painters": {
         "name": "Painters",
@@ -28,7 +27,7 @@ MODELS = {
         "target_image_size": (256, 256),
         "layers": ["penultimate"],
         "order": 4,
-        "batch_size": 20,
+        "batch_size": 500,
     },
     "deeploc": {
         "name": "DeepLoc",
@@ -36,7 +35,7 @@ MODELS = {
         "target_image_size": (64, 64),
         "layers": ["penultimate"],
         "order": 5,
-        "batch_size": 20,
+        "batch_size": 500,
     },
     "vgg16": {
         "name": "VGG-16",
@@ -44,7 +43,7 @@ MODELS = {
         "target_image_size": (224, 224),
         "layers": ["penultimate"],
         "order": 2,
-        "batch_size": 15,
+        "batch_size": 500,
     },
     "vgg19": {
         "name": "VGG-19",
@@ -52,7 +51,7 @@ MODELS = {
         "target_image_size": (224, 224),
         "layers": ["penultimate"],
         "order": 3,
-        "batch_size": 15,
+        "batch_size": 500,
     },
     "openface": {
         "name": "openface",
@@ -61,7 +60,7 @@ MODELS = {
         "target_image_size": (256, 256),
         "layers": ["penultimate"],
         "order": 6,
-        "batch_size": 20,
+        "batch_size": 500,
     },
     "squeezenet": {
         "name": "SqueezeNet",
@@ -90,10 +89,18 @@ class ImageEmbedder:
 
     Examples
     --------
+    >>> import Orange
     >>> from orangecontrib.imageanalytics.image_embedder import ImageEmbedder
-    >>> image_file_paths = [...]
+
+    >>> # embedding from list of paths
+    >>> image_file_paths = ['image001.jpg', 'image001.jpg']
     >>> with ImageEmbedder(model='model_name') as emb:
     ...    embeddings = emb(image_file_paths)
+
+    >>> # embedding from orange tabl
+    >>> table = Orange.data.Table('Table_with_image_path.csv')
+    >>> with ImageEmbedder(model='model_name') as emb:
+    ...    embeddings = emb(table, col="image_path_column")
     """
 
     _embedder = None
@@ -139,6 +146,7 @@ class ImageEmbedder:
                 self._model_settings["batch_size"],
                 self.server_url,
                 "image",
+                self._model_settings["target_image_size"]
             )
 
     def __call__(
@@ -203,31 +211,7 @@ class ImageEmbedder:
             Optional callback - function that is called for every embedded
             image and is used to report the progress.
         """
-        file_paths_attr = data.domain[col]
-        file_paths = data[:, file_paths_attr].metas.flatten()
-        origin = file_paths_attr.attributes.get("origin", "")
-        if (
-            urlparse(origin).scheme in ("http", "https", "ftp", "data")
-            and origin[-1] != "/"
-        ):
-            origin += "/"
-
-        assert file_paths_attr.is_string
-        assert file_paths.dtype == np.dtype("O")
-
-        file_paths_mask = file_paths == file_paths_attr.Unknown
-        # make sure that not defined images in not embedded
-        file_paths[file_paths_mask] = None
-        for i, a in enumerate(file_paths):
-            urlparts = urlparse(a)
-            if a is None:
-                continue
-            if urlparts.scheme not in ("http", "https", "ftp", "data"):
-                if urlparse(origin).scheme in ("http", "https", "ftp", "data"):
-                    file_paths[i] = urljoin(origin, a)
-                else:
-                    file_paths[i] = os.path.join(origin, a)
-
+        file_paths = extract_paths(data, data.domain[col])
         embeddings_ = self._embedder.embedd_data(file_paths, callback)
         return ImageEmbedder.prepare_output_data(data, embeddings_)
 
@@ -259,8 +243,7 @@ class ImageEmbedder:
         Table with added embeddings to data.
         """
         new_attributes = [
-            ContinuousVariable.make("n{:d}".format(d))
-            for d in range(embeddings_.shape[1])
+            ContinuousVariable("n{:d}".format(d)) for d in range(embeddings_.shape[1])
         ]
         # prevent embeddings to be shown in long drop-downs in e.g. scatterplot
         for a in new_attributes:
@@ -272,7 +255,8 @@ class ImageEmbedder:
             embedded_images.domain.metas,
         )
         table = embedded_images.transform(domain_new)
-        table[:, new_attributes] = embeddings_
+        with table.unlocked(table.X):  # writing to fresh part, can be unlocked
+            table[:, new_attributes] = embeddings_
 
         return table
 
@@ -325,23 +309,6 @@ class ImageEmbedder:
 
         return embedded_images, skipped_images, num_skipped
 
-    @staticmethod
-    def filter_image_attributes(data: Table) -> List[Variable]:
-        """
-        Filter attributes that have image paths data.
-
-        Parameters
-        ----------
-        data
-            Table with data
-
-        Returns
-        -------
-        List of variables that are attributes with image paths.
-        """
-        metas = data.domain.metas
-        return [m for m in metas if m.attributes.get("type") == "image"]
-
     def clear_cache(self) -> None:
         """
         Function clear cache for the selected embedder. If embedder is loaded
