@@ -1,6 +1,7 @@
 import enum
 import itertools
 import logging
+import os
 from collections import namedtuple
 from concurrent.futures import Future
 from itertools import zip_longest
@@ -8,7 +9,7 @@ from itertools import zip_longest
 import Orange.data
 import numpy as np
 from AnyQt.QtCore import (
-    Qt, QEvent, QSize, QSizeF, QRectF, QPointF, QUrl, QDir
+    Qt, QEvent, QSize, QSizeF, QRectF, QPointF, QUrl
 )
 from AnyQt.QtCore import pyqtSignal as Signal, pyqtSlot as Slot
 from AnyQt.QtGui import (
@@ -19,8 +20,13 @@ from AnyQt.QtWidgets import (
     QGraphicsScene, QGraphicsView, QGraphicsWidget, QGraphicsItem,
     QGraphicsRectItem,
     QGraphicsLinearLayout,
-    QGraphicsGridLayout, QSizePolicy, QApplication, QStyle, QShortcut,
-    QFormLayout, QLabel)
+    QGraphicsGridLayout,
+    QSizePolicy,
+    QApplication,
+    QStyle,
+    QShortcut,
+    QLabel,
+)
 from Orange.widgets import widget, gui, settings
 from Orange.widgets.settings import ContextSetting
 from Orange.widgets.utils.annotated_data import (
@@ -31,8 +37,12 @@ from Orange.widgets.utils.overlay import proxydoc
 from Orange.widgets.widget import Input, Output, OWWidget, Msg
 
 from orangecontrib.imageanalytics.image_grid import ImageGrid
-from orangecontrib.imageanalytics.widgets.owimageviewer import (
-    ImageLoader, Preview)
+from orangecontrib.imageanalytics.utils.image_utils import (
+    extract_image_path,
+    filter_image_attributes,
+)
+from orangecontrib.imageanalytics.widgets.owimageviewer import ImageLoader, Preview
+
 
 _log = logging.getLogger(__name__)
 
@@ -112,7 +122,6 @@ class OWImageGrid(widget.OWWidget):
             tooltip="Attribute with image filenames",
             callback=self.change_image_attr,
             contentsLength=12,
-            addSpace=True,
         )
 
         # cell fit (resize or crop)
@@ -122,25 +131,24 @@ class OWImageGrid(widget.OWWidget):
 
         self.gridSizeBox = gui.vBox(self.controlArea, "Grid size")
 
-        form = QFormLayout(
-            labelAlignment=Qt.AlignLeft,
-            formAlignment=Qt.AlignLeft,
-            fieldGrowthPolicy=QFormLayout.AllNonFixedFieldsGrow,
-            verticalSpacing=10
-        )
-
         self.colSpinner = gui.spin(
-            self.gridSizeBox, self, "columns", minv=2, maxv=40,
-            callback=self.update_size)
+            self.gridSizeBox,
+            self,
+            "columns",
+            minv=2,
+            maxv=40,
+            label="Columns",
+            callback=self.update_size,
+        )
         self.rowSpinner = gui.spin(
-            self.gridSizeBox, self, "rows", minv=2, maxv=40,
-            callback=self.update_size)
-
-        form.addRow("Columns:", self.colSpinner)
-        form.addRow("Rows:", self.rowSpinner)
-
-        gui.separator(self.gridSizeBox, 10)
-        self.gridSizeBox.layout().addLayout(form)
+            self.gridSizeBox,
+            self,
+            "rows",
+            minv=2,
+            maxv=40,
+            label="Rows",
+            callback=self.update_size,
+        )
 
         gui.button(
             self.gridSizeBox, self, "Set size automatically",
@@ -153,16 +161,13 @@ class OWImageGrid(widget.OWWidget):
             self.label_box, self, "label_attr",
             tooltip="Show labels",
             callback=self.update_size,
-            addSpace=True,
             model=self.label_model
         )
 
         gui.rubber(self.controlArea)
 
         # auto commit
-        self.autoCommitBox = gui.auto_commit(
-            self.controlArea, self, "auto_commit", "Apply",
-            checkbox_label="Apply automatically")
+        gui.auto_commit(self.buttonsArea, self, "auto_commit", "Apply")
 
         self.image_grid = None
         self.cell_fit = 0
@@ -198,20 +203,12 @@ class OWImageGrid(widget.OWWidget):
             self.allAttrs = (domain.class_vars + domain.metas +
                              domain.attributes)
 
-            self.stringAttrs = [a for a in domain.metas if a.is_string]
+            self.stringAttrs = filter_image_attributes(data)
             self.domainAttrs = len(domain.attributes)
 
-            self.stringAttrs = sorted(
-                self.stringAttrs,
-                key=lambda attr: 0 if "type" in attr.attributes else 1
-            )
-
-            indices = [i for i, var in enumerate(self.stringAttrs)
-                       if var.attributes.get("type") == "image"]
-            if indices:
-                self.imageAttr = indices[0]
-
             self.imageAttrCB.setModel(VariableListModel(self.stringAttrs))
+            if self.stringAttrs:
+                self.imageAttr = 0
 
             # set label combo labels
             self.label_model.set_domain(domain)
@@ -225,6 +222,7 @@ class OWImageGrid(widget.OWWidget):
                 self.setup_scene()
             else:
                 self.Warning.no_valid_data()
+        self.commit.now()
 
     @Inputs.data_subset
     def set_data_subset(self, data_subset):
@@ -272,12 +270,11 @@ class OWImageGrid(widget.OWWidget):
                 thumbnail.instance = inst
                 self.thumbnailView.addThumbnail(thumbnail)
 
-                if not np.isfinite(inst[attr]) or inst[attr] == "?":
-                    # skip missing
-                    future, url = None, None
-                else:
-                    url = self.url_from_value(inst[attr])
-                    thumbnail.setToolTip(url.toString())
+                path = extract_image_path(inst, attr)
+                future, url = None, None
+                if path is not None:
+                    url = QUrl.fromLocalFile(path) if os.path.exists(path) else QUrl(path)
+                    thumbnail.setToolTip(path)
                     self.nonempty.append(i)
 
                     if url.isValid() and url.isLocalFile():
@@ -353,20 +350,6 @@ class OWImageGrid(widget.OWWidget):
                 self.Warning.incompatible_subset()
         self.apply_subset()
 
-    def url_from_value(self, value):
-        base = value.variable.attributes.get("origin", "")
-        if QDir(base).exists():
-            base = QUrl.fromLocalFile(base)
-        else:
-            base = QUrl(base)
-
-        path = base.path()
-        if path.strip() and not path.endswith("/"):
-            base.setPath(path + "/")
-
-        url = base.resolved(QUrl(str(value)))
-        return url
-
     def cancel_all_futures(self):
         for item in self.items:
             if item.future is not None:
@@ -452,14 +435,22 @@ class OWImageGrid(widget.OWWidget):
             self.selection[indices] = 1
 
         self.update_selection()
-        self.commit()
+        self.commit.deferred()
 
+    @gui.deferred
     def commit(self):
-        if self.data:
+        # if data are not valid - not in grid do not output
+        if self.data and len(self.items) > 0:
+            if self.selection is None:
+                self.selection = np.zeros(len(self.items), dtype=np.uint8)
             # add Group column (group number)
             self.Outputs.selected_data.send(
-                create_groups_table(self.image_grid.image_list, self.selection,
-                                    False, "Group"))
+                create_groups_table(
+                    self.image_grid.image_list, self.selection, False, "Group"
+                )
+                if np.any(self.selection > 0)
+                else None
+            )
 
             # filter out empty cells - keep indices of cells that contain images
             # add Selected column
@@ -473,7 +464,6 @@ class OWImageGrid(widget.OWWidget):
                     self.image_grid.image_list[self.nonempty],
                     np.nonzero(self.selection[self.nonempty]))
             self.Outputs.data.send(out_data)
-
         else:
             self.Outputs.data.send(None)
             self.Outputs.selected_data.send(None)
@@ -483,7 +473,7 @@ class OWImageGrid(widget.OWWidget):
             pen, brush = self.compute_colors()
 
             for s, item, p, b in zip(self.selection, self.items, pen, brush):
-                item.widget.setSelected(s > 0)
+                item.widget.setSelected(bool(s > 0))
                 item.widget.setSelectionColor(p, b)
 
     # Adapted from Scatter Plot Graph (change brush instead of pen)
@@ -524,9 +514,10 @@ class OWImageGrid(widget.OWWidget):
 
         if self._errcount + self._successcount == count:
             attr = self.stringAttrs[self.imageAttr]
-            if self._errcount == count and "type" not in attr.attributes:
-                self.error("No images found! Make sure the '%s' attribute "
-                           "is tagged with 'type=image'" % attr.name)
+            if self._errcount == count:
+                self.error(f"No images found for column '{attr.name}'! "
+                           f"Make sure to usa a column with valid image path "
+                           f"name or urls.")
 
     def onDeleteWidget(self):
         self.cancel_all_futures()
@@ -718,7 +709,7 @@ class GraphicsThumbnailWidget(QGraphicsWidget):
         if self._size != size:
             self._size = QSizeF(size)
             if self.label is not None:
-                self.label.setFixedWidth(size.width())
+                self.label.setFixedWidth(int(size.width()))
             self._updatePixmapSize()
 
     def paint(self, painter, option, widget=0):
