@@ -7,7 +7,7 @@ from Orange.data import StringVariable, Table, Domain
 def load_images():
     path = os.path.join(os.path.dirname(os.path.realpath(__file__)),
                         "test_images")
-    files = [[x] for x in os.listdir(path)]
+    files = [[x] for x in sorted(os.listdir(path))]
 
     image_var = StringVariable(name="Images")
     image_var.attributes["type"] = "image"
